
 
; export symbols
            XDEF Entry     ;export 'Entry' symbol
            ABSENTRY Entry ;for absolute assembly: 
			               ;mark this as application entry point
           
;include derivative specific macros
            Include 'qy4_registers.inc'
;QF4 EValuation Board Software
;
;######################################################################
;#                                                             
;#               TRANSMISSION FRAME COMPOSITION                          
;#                                                             
;#  NNNNNNNN-RRRRRRRR-DGGGTTTT-IIIIIIII-IIIIIIII-IIIIIIII             
;# | EXTRA  | --CRC--| codice |--------ID  NUMBER--------                                                                              #
;# I=ID NUMBER -> FIX IDENTIFICATION CODE (3 BYTES)                
;# G=GROUP     -> SELECTED GROUP                                  
;# D=DIRECTION -> 1=INCREASE 0=DECREASE                          
;# M=MODE      -> 1=Toggle 0=Normal                            
;# T=BUTTON    -> BUTTON SELECTED CODE                         
;# R=CRC       ->  CRC                                        
;# N=UNUSED    -> THEY MUST BE  0 (Low) To Initiate comnication
;#                                                           
;#                                                           
;#                         TEMPORIZATIONS                                                                   #
;# Duration Time of the sync signal = 6ms                          
;# Duration Time of a Bit           = 2ms                          
;# Bit 0 -> LowTime=1.33ms HighTime=0.66ms                         
;# Bit 1 -> LowTime=0.66ms HighTime=1.33ms                         
;#                                                           
;#######################################################################


DDRAInit      equ     2                    ;Configure PTA1 DATA to be an Output
PortAInit     equ     0                    ;Initialize DATA low

DATA          equ     1
; Porta B

DDRBInit      equ     1                    ;Initialize PTB0, ENABLE
PortBInit     equ     0                    ;

ENABLE        equ     0

              ORG     $80

Buffer0       DS      1                    ;Control Byte
Buffer1       DS      1                    ;ID0
Buffer2       DS      1                    ;ID1
Buffer3       DS      1                    ;ID2
Buffer4       DS      1                    ;Counter Data
Buffer5       DS      1                    ;RC0
Buffer6       DS      1                    ;RC1
CRC           DS      1                    ;CRC  

BitCount      DS      1                      
LastRow       DS      1                
LastCol       DS      1                
TxCnt         DS      1                

Counter       DS      1

MinTx         equ     $03                  ;Minimum number of frame transmitted
FrameDly      equ     $20                  ;Delay between frames (771*FrameDly cyc)



              Org     $EE00
Entry:
Start
              mov     #%00000011,CONFIG1   ;Diasble COP and 
                                           ;enable Stop instruction
              rsp                          ;Init stack pointer

              mov     #DDRAInit,DDRA       ;Init PORTA
              mov     #PortAInit,PORTA                
              mov     #DDRBInit,DDRB       ;Init PORTB
              mov     #PortBInit,PORTB                      
              mov     #$00,Counter         ;Init Counter Variable
Sleep
              bclr    ENABLE,PORTB         ;Stop PLL of Tango

              inc     Counter              ;Increment Counter variable
              lda     Counter
                                
;TRANSMISSION BUFFER CREATED HERE

               ;Init Transmission Buffer
SetKey
              sta     Buffer4              ;Place Counter Data in Buffer
              clr     Buffer0              ;
              lda     ID0                  ;Place ID constants in Buffer
              sta     Buffer1              ;
              lda     ID1                  ;
              sta     Buffer2              ;
              lda     ID2                  ;
              sta     Buffer3              ;
                                           ;


              lda     #$55                 ;
             
              sta     Buffer6              ;Load Buffer 5 and 6 with a constant
              sta     Buffer5              ;These are used to encode the data
              
              
              ;DATA IS ENCODED HERE            
              
              lda     Buffer6              ;Buffer 6 is used to determine how many
              and     #$0F                 ;Rotates should Occur
              beq     NoRotate             ;
              tax                          ;
NextRotate
              lda     Buffer5              ;
              rora                         ;
              ror     Buffer0              ;
              ror     Buffer1              ;
              ror     Buffer2              ;
              ror     Buffer3              ;
              ror     Buffer4              ;
              ror     Buffer5              ;
              dbnzx   NextRotate           ;
NoRotate
              ldhx    #Buffer0             ;
              lda     ,X                   ;
              eor     1,X                  ;
              sta     1,X                  ;
              eor     2,X                  ;
              sta     2,X                  ;
              eor     3,X                  ;
              sta     3,X                  ;
              eor     4,X                  ;
              sta     4,X                  ;
              eor     5,X                  ;
              sta     5,X                  ;


              ; Calculate CRC

              clr     CRC                
              ldhx    #Buffer6            
NextCRCByte
              mov     #$08,BitCount            
NextCRCBit
              lda     0,X                
              eor     CRC                
              lsr     CRC                
              and     #$01                
              beq     NoXor                
              lda     #%10001100            
              eor     CRC                
              sta     CRC                
NoXor
              lda     0,X                
              rora                      
              ror     0,X                
              dbnz    BitCount,NextCRCBit      
              decx                      
              cpx     #Buffer0-1            
              bne     NextCRCByte            
              lda      CRC                
              ror     CRC                
              rora                      
              nsa                      
              sta     CRC                

;#######################################################################
;#                              
;#                           Data Timing                                           
;#                                                                      
;# Timing is derived from Tango DataClock signal (Tango Xtal/64) that is        
;# 212Khz@13.560Mhz (4.717 us period).                                        
;# Tango DataClock is used as Timer Module Clock by MCU. When external clock        
;# is used by Timer Module the prescaler is not available and the clock is         
;# directly to the timer counter.                                            
;#                                                                       
;#######################################################################

              ;Transmission Loop

              mov     #DDRAInit,DDRA                
              mov     #PortAInit,PORTA                
              mov     #MinTx,TxCnt                
 
              bset    ENABLE,PORTB         ;Enable Tango
 
              mov     #%00110111,TSC       ;Stop and reset Timer
NextFrame
              bset    DATA,PORTA                
              ldx     #FrameDly                
              clra                            
FrameBlank
              clr     TSC1                            
              dbnza   $                      
              dbnzx   FrameBlank                
              mov     #%00110111,TSC       ;Stop and reset Timer
              bclr    DATA,PORTA       
 
              mov     #%00010010,TSC1      ;
 
              ldhx    #$0300               ;Timer modulo=$0300 (3.622ms)
              sthx    TMODH                ;
              ldhx    #$0180               ;Compare Value=$0180 (0.603ms)
              sthx    TCH1H                ;
              bclr    7,TSC1               ;Clear Flag
              mov     #%00011110,TSC1      ;Set On Compare and Toggle on Overflow
              bclr    7,TSC                ;Clear Flag
              bclr    5,TSC                ;Start Timer
              brclr   7,TSC,*              ;Wait
              mov     #%00110111,TSC       ;Stop Timer
              ldhx    #$00D2               ;Timer modulo=$00D2 (990us)
              sthx    TMODH                ;
              clr     TCH1H                ;
              lda     #$46                 ;
              sta     TCH1L                ;Compare Value=1/3 bit time
              ldhx    #CRC                 ;HX points to CRC
              psha                         ;Space in stack for bit counter
              mov     #%00011110,TSC1      ;Set On Compare and Toggle on Overflow
              bclr    5,TSC                ;Start Timer
NextTxByte
              lda     #$08                 ;Init Bit Counter
              sta     1,SP                 ;
NextTxBit
                      lda      0,X         ;
              rora                         ;
              ror     0,X                  ;
              clr     TCH1H                ;
              lda     #$46                 ;Compare Value=1/3 bit time if bit=1
              bcs     Bit1                 ;
              lsla                         ;Compare Value=2/3 bit time if bit=0
Bit1
              sta     TCH1L                ;
              brclr   7,TSC,*              ;Wait
              bclr    7,TSC                ;Clear Flag
              dbnz    1,SP,NextTxBit       ;
              decx                         ;
              cpx     #Buffer0-1           ;
              bne     NextTxByte           ;
              mov     #%00110111,TSC       ;Stop Timer
              pula                         ;Free stack
              clra                         ;
              dbnza   $                    ;
              tst     TxCnt                ;
              beq     End_Trans            ;
              dbnz    TxCnt,NextFrame      ;

End_Trans              
              bclr    ENABLE,PORTB         ;
              mov     #DDRAInit,DDRA       ;
              mov     #PortAInit,PORTA     ;
              jmp     Sleep               
Dummy
              
              rti
                
                      ;ID CONSTANTS
ID0           FCB     $55 
ID1           FCB     $AA
ID2           FCB     $22

              org     $FFDE
              fdb     Dummy                ;ADC vector
              fdb     Dummy                ;Keyboard vector
              org     $FFF2
              fdb     Dummy                ;Timer overflow vector
              org     $FFF4
              fdb     Dummy                ;Timer channel 1 vector
              fdb     Dummy                ;Timer channel 0 vector
              org     $FFFA
              fdb     Dummy                ;IRQ vector
              fdb     Dummy                ;SWI vector
              fdb     Entry
